% Model Reference Adaptive System
clc; clear all; close all;
% Store starting time
t0=fix(clock);

global wi0 wi vi0 vi zi Lri Nui xi eu r
global wf0 wf vf0 vf zf Lrf Nuf xf y yh u J

% Problem constants..
k1=0.34366; k2=-0.1534069; k3=-2.286928e-3; k4=3.5193358e-4; k5=0.2280595;...
k6=-0.105185;
Ts=0.25; % Sampling time
Ntime=500; % Number of samples
Nui=8; % Number of hidden units in inverse model
Nuf=5; % Number of hidden units in forward model

y(1)=0; y(2)=0;

% Defining the desired response
sysc = tf(42,[1 11 30]);
sysd = c2d(sysc,Ts,'zoh');
steps; 
% Use stairs as a different input
% stairs;
y_ref=lsim(sysd,r); % Simulate response of sysd for input r


% Plant response before training
for i=2: (Ntime-1)
     y(i+1) = r(i)*k5 + k1*y(i) + k2*y(i-1) + k3*sign(y(i))*y(i)*y(i) ...
              + k4*sign(y(i))*y(i-1)*y(i-1)+k6;
end
y_before=y;


% Learning rates
Lri=0.99;  Lrf=0.512;
y(1:2)=0;  u(1)=0;
 
% Initial values of forward and inverse model are chosen randomly
% and stored in Initial_values.mat
load Initial_values;

% Starting the MRAS iterations 

for k=2: Ntime-1
    if mod(k,100)==0
        fprintf('Processing data point No: %d \n',k);
    end
    % Train the inverse model
    Update_IM(k);
    
    % Simulate inverse model to get control input
    u(k)=Eval_IM(k);
    
    % Train the forward model
    Update_FM(k);

    % Simulate the forward model to get jacobian
    % and predicted output
    [yh(k+1),J(k+1)]=Eval_FM(k);

    % Calculating the plant output to the above control input U(k)
    y(k+1) = u(k)*k5 + k1*y(k) + k2*y(k-1) + k3*sign(y(k))*y(k)*y(k) ...
        + k4*sign(y(k))*y(k-1)*y(k-1)+k6;

    % Calculating the error
    e(k+1) = y_ref(k+1)-yh(k+1);
    eu(k+1)=e(k+1)*J(k+1);
end


figure(1);
plot(r);
hold on;plot(y_ref,'r');

figure(2);
plot(y,'r');
hold on;  plot(yh,'k');
xlabel('Time step');
ylabel('Output');
title('System output Vs Forward model output');
legend('Actual','Forward Model');

figure(3);
plot(y,'r');
hold on;  plot(y_ref,'b');
xlabel('Time step');
ylabel('Output');
title('Reference output Vs Actual plant output');
legend('Actual','Reference');

% Calculate time taken for execution
str=time_taken(t0)