% Function approximation using anfis
clc; clear all; close all;

% The given function y=f(x1,x2)
% Training data: input pattern and output pattern
% x1 and x2 spaced between -10 and 10 with 11 points

x1=-10:2:10;
x2=-10:2:10;
[xx1 xx2]=meshgrid(x1,x2);

tmp1 = sin(xx1)./(xx1);
index = find(isnan(tmp1)==1);
tmp1(index) = ones(size(index));

tmp2 = sin(xx2)./(xx2);
index = find(isnan(tmp2)==1);
tmp2(index) = ones(size(index));

yy = tmp1.*tmp2;
traindata = [xx1(:) xx2(:) yy(:)];

% Create an initial fis
nmf=4; % Number of membership functions
% genfis1 is used to generate sugeno type fis
fis_initial = genfis1(traindata,nmf,'trimf');

% Train the fis for optimal membership function parameters
epochs=100;
ss = 0.1;
[fis_trained train_error step_size] = ...
	anfis(traindata, fis_initial, [epochs nan ss]);

% Output of trained anfis
yycap = evalfis([xx1(:) xx2(:)],fis_trained);
yycap=reshape(yycap,11,11);
% Plot training inputs & output
figure(1);
mesh(xx1,xx2,yy);
% Plot output given by trained anfis
figure(2);
mesh(xx1,xx2,yycap);
% Plot of error
figure(3)
plot(1:epochs,train_error);

% Plot of initial membership functions
figure(4)
subplot(211); plotmf(fis_initial,'input',1);
subplot(212); plotmf(fis_initial,'input',2);
% Plot of trained membership function
figure(5)
subplot(211); plotmf(fis_trained,'input',1);
subplot(212); plotmf(fis_trained,'input',2);

% MLP (2-18-1) output
nn_train_xx=[xx1(:)';xx2(:)'];
nn_train_yy=yy(:)';
net=newff(minmax(nn_train_xx),[18 1],{'tansig','purelin'},'traingdx'); 
net.trainparam.goal=0.005;
net.trainparam.epochs=500;
net.trainparam.show=50;
[net,tr]=train(net,nn_train_xx,nn_train_yy);
nn_train_yycap=sim(net,nn_train_xx);
nn_train_yycap=reshape(nn_train_yycap,11,11);
figure(6)
mesh(xx1,xx2,nn_train_yycap);